require 'exifr'
require 'pathname'
require 'time'
require 'digest'

module Roopap
  class Organizer
    def initialize
      @count = 0
    end

    def perform(source, dest)
      Pathname.new(source).each_child do |c|
        src_file_path =  c.to_s
        next unless jpg_file?(src_file_path)
        target_path = dest_for(src_file_path, dest)
        copy(src_file_path, target_path)
        inc_success_number
      end
    end

    def organized_file_count
      @count
    end

    private
    def jpg_file?(path)
      begin
        EXIFR::JPEG.new(path)
        true
      rescue EXIFR::MalformedJPEG
        false
      end
    end

    def dest_for(filepath, root)
      photo = EXIFR::JPEG.new(filepath)
      time = get_time(photo)
      dir_name = time.utc.strftime('%Y-%m')
      dir_path = File.join(root, dir_name)
      prepare_dir(dir_path)
      filename = gen_filename(filepath)
      File.join(dir_path, filename)
    end

    def get_time(photo)
      time = photo.date_time_original
      time = photo.date_time if (time.nil? || time == '')
      time = Time.parse(time) if time.instance_of?(String)
      time
    end

    def gen_filename(filepath)
      "#{Digest::MD5.hexdigest(File.read(filepath))[0..9]}.jpg"
    end

    def prepare_dir(path)
      FileUtils.mkdir_p(path)
    end

    def copy(filepath, dest_path)
      FileUtils.cp(filepath, dest_path)
    end

    def inc_success_number
      @count = @count + 1
    end
  end
end
